## МенеджерСущностей

### ПриСозданииОбъекта

```bsl
// Конструктор объекта МенеджерСущностей.
//
// Параметры:
//   ТипКоннектора - Тип - Тип класса, реализующего интерфейс Коннектор.
//   СтрокаСоединения - Строка - Строка соединения к БД, к которой подключается коннектор.
//   ППараметрыКоннектора - Массив - Массив дополнительных параметров коннектора. Содержимое произвольное.
//
Процедура ПриСозданииОбъекта(Знач ТипКоннектора, Знач СтрокаСоединения = "", Знач ППараметрыКоннектора = Неопределено) 
```

### ДобавитьКлассВМодель

```bsl
// Регистрирует переданный тип класса-сценария в модели данных.
//
// Параметры:
//   ТипСущности - Тип - Тип для добавления в модель
//
Процедура ДобавитьКлассВМодель(ТипСущности) 
```

### Инициализировать

```bsl
// Запускает процессы инициализации коннектора и таблиц БД.
//
Процедура Инициализировать() 
```

### Сохранить

```bsl
// Сохраняет сущность в БД.
//
// Параметры:
//   Сущность - Произвольный - Объект (экземпляр класса, зарегистрированного в модели) для сохранения в БД.
//
Процедура Сохранить(Сущность) 
```

### Получить

```bsl
// Осуществляет поиск сущностей переданного типа по идентификатору.
//
// Параметры:
//   ТипСущности - Тип - Тип искомой сущности.
//   Отбор - Произвольный - Отбор для поиска.
//     Если параметр не задан или равен "Неопределено", то возвращаются все найденные сущности указанного типа.
//     Если параметр имеет тип "Соответствие", то каждое значение соответствия преобразуется к условию поиска
//      ИмяПоля = ЗначениеПоля, где ИмяПоля - ключ элемента соответствия, ЗначениеПоля - значение элемента соответствия.
//     Если параметр имеет тип "Массив", то каждое элемент массива должен иметь тип "ЭлементОтбора".
//       Каждый элемент отбора преобразуется к условию поиска. В качестве "ПутьКДанным" указываются имена полей.
//     Если параметр имеет тип "ЭлементОтбора", то элемент отбора преобразуется к условию поиска.
//       В качестве "ПутьКДанным" указываются имена полей.
//
//  Возвращаемое значение:
//   Массив - Массив найденных сущностей. В качестве элементов массива выступают
//     экземпляры класса с типом, равным переданному "ТипуСущности", с заполненными значениями полей.
//
Функция Получить(ТипСущности, Отбор = Неопределено) 
```

### ПолучитьОдно

```bsl
// Осуществляет поиск сущности переданного типа по идентификатору.
//
// Параметры:
//   ТипСущности - Тип - Тип искомой сущности.
//   Отбор - Произвольный - Отбор для поиска.
//     Если параметр не задан или равен "Неопределено", то возвращаются все найденные сущности указанного типа.
//     Если параметр имеет тип "Соответствие", то каждое значение соответствия преобразуется к условию поиска.
//      ИмяПоля = ЗначениеПоля, где ИмяПоля - ключ элемента соответствия, ЗначениеПоля - значение элемента соответствия.
//     Если параметр имеет тип "Массив", то каждое элемент массива должен иметь тип "ЭлементОтбора".
//       Каждый элемент отбора преобразуется к условию поиска. В качестве "ПутьКДанным" указываются имена полей.
//     Если параметр имеет тип "ЭлементОтбора", то элемент отбора преобразуется к условию поиска.
//       В качестве "ПутьКДанным" указываются имена полей.
//     Любой другой тип интерпретируется как поиск по &Идентификатору.
//
//  Возвращаемое значение:
//   Произвольный - Если сущность была найдена, то возвращается экземпляр класса с типом, равным переданному
//     "ТипуСущности", с заполненными значениями полей. Иначе возвращается "Неопределено".
//
Функция ПолучитьОдно(ТипСущности, Знач Отбор = Неопределено) 
```

### Удалить

```bsl
// Выполняет удаление сущности из базы данных.
// Сущность должна иметь заполненный идентификатор.
//
// Параметры:
//   Сущность - Произвольный - Удаляемая сущность
//
Процедура Удалить(Сущность) 
```

### Закрыть

```bsl
// Выполняет очистку полную данных библиотеки.
// Дополнительно посылает всем используемым коннекторам запросы на закрытие соединения.
//
Процедура Закрыть() 
```

### НачатьТранзакцию

```bsl
// Посылает коннектору запрос на начало транзакции.
//
Процедура НачатьТранзакцию() 
```

### ЗафиксироватьТранзакцию

```bsl
// Посылает коннектору запрос на фиксацию транзакции.
//
Процедура ЗафиксироватьТранзакцию() 
```

### ОтменитьТранзакцию

```bsl
// Посылает коннектору запрос на отмену транзакции.
//
Процедура ОтменитьТранзакцию() 
```

### ПолучитьКоннектор

```bsl
// Возвращает текущий активный коннектор.
//
//  Возвращаемое значение:
//   АбстрактныйКоннектор - Возвращает экземпляр коннектора. Конкретная реализация определяется параметром
//      ТипКоннектора при вызове конструктора МенеджерСущностей.
//
Функция ПолучитьКоннектор() 
```

### ПолучитьХранилищеСущностей

```bsl
// Получает ХранилищеСущностей, привязанное к переданному типу сущности.
//
// Параметры:
//   ТипСущности - Тип - Тип сущности, зарегистрированный в Модели
//
//  Возвращаемое значение:
//   ХранилищеСущностей - Хранилище сущностей, привязанное к переданному типу сущности.
//
Функция ПолучитьХранилищеСущностей(ТипСущности) 
```

### ПолучитьПулСущностей

```bsl
// @internal
// Для служебного пользования.
//
// Возвращает пул сущностей из хранилища сущностей, привязанного к переданному типу сущности.
//
// Параметры:
//   ТипСущности - Тип - Тип сущности, зарегистрированный в Модели.
//
//  Возвращаемое значение:
//   Соответствие - Пул сущностей.
//
Функция ПолучитьПулСущностей(ТипСущности) 
```

